% Simplified wind turbine model
%
% (c) D.J.Leith, 2002
%
% This software may be freely distributed provided it is not modified (in particular this notice must not 
% be removed) or used for commercial purposes without permission in writing.  
%
% Initialisation for wind turbine simulation
global PointA PointB wmean
global gamma Radius
global RatGB StfHs DampHs IneHs StfLs StfLs DampLs IneLs GamLS Effy RatedP
global OmLdic TqlsIC OmlsIC tau1 tau2 tau3 kw
global g1 g2 N Kv Kp wref

% run time parameters
%wmean=22;

% machine parameters
pi     = 3.14152;
rho    = 1.225;       %" kg/m3 "
IneLs  = 1.029e6;  %" kg-m2 - inertia of low speed shaft "
IneHs  = 42.82;     %" kg-m2 - inertia of high speed shaft "
StfLs  = 1.0106e8;  %" Nm/rad - stiffness of Lss "
StfHs  = 4.85e6;       %" Nm/rad - stiffness of Hss "
RatGB  = 58.0;         %" Gear box ratio "
DampHs= 4.5112;        %" Nm/rad/s  Shaft Damping"
DampLs= 1.5176e4;    %" Nm/rad/s  Shaft Damping"
GamLS = 2.6538e5;      %"Nm/rad/s additional damping"
wref = 2.72271;        %"Rad/s above rated speed setpoint"
Effy   = 1.0;      %" %efficent-generator "
RatedP = 1.0e6;        %" Watts - rated power "
Radius = 26.0;         %" rotor radius - metres "
kw=0.05;
% alternative versions
g1=DampLs; g2=DampHs; gdash=GamLS;
k1=StfLs/(1+StfLs/StfHs/RatGB/RatGB); k2=StfHs;
I1=IneLs; I2=IneHs;
N=RatGB;
Kv=3200; Kp=29500;

% spatial filter parameters
beta = gamma*Radius/wmean; a = 0.55;   gamma = 1.3;  arun = 0.1;           %" Average windspeed time constant "
sqrta = sqrt(a); sqrt2 = sqrt(2.);
tau1 = beta/sqrt2;
tau2 = sqrta*beta/sqrt2;
tau3 = beta/sqrta;

%" Point Spectra A and B parameters - (Not valid below 11m/s)"
%"20% turbulence"
PointA=[ ...
	0 0.050;11 0.063;12 0.068;13 0.074;14 0.080;...
	15 0.086;16 0.092;17 0.097;18 0.103;19 0.108;...
	20 0.114;21 0.119;22 0.125;23 0.131;100 0.131];
PointB=[...
	0 0.619;11 0.781;12 0.885;13 1.00;14 1.12;...
	15 1.240;16 1.370;17 1.500;18 1.63;19 1.77;...
	100 1.77];
Awind=interp1(PointA(:,1),PointA(:,2), wmean);
Bwind=interp1(PointB(:,1),PointB(:,2), wmean);

% initial conditions
OmLdic=RatGB*wref;
TqlsIC=219501.0; %"i.e.steady-state in mode 2 at wmean=10"
OmlsIC=wref;

return

