% Analyse performance of a wind turbine controller implemented in simulink model wind_turbine2.mdl
%
% (c) D.J.Leith, 2002
%
% This software may be freely distributed provided it is not modified (in particular this notice must not 
% be removed) or used for commercial purposes without permission in writing.  
%

fprintf('***Performance Analysis ***\n')
fprintf('Running simulation model to collect data.\n')
fprintf('Mean wind speed of 5 m/s ...\n')
global wmean
wmean=5;init;
sim('wind_turbine2')
wind5=wind(1000:30000,2); power5=power(1000:30000,2); tqld5=tqld(1000:30000,2); omgls5=omgls(1000:30000,2); pitchvel5=pitchvel(1000:30000,2);
fprintf('Mean wind speed of 8 m/s ...\n')
wmean=8;init;
sim('wind_turbine2')
wind8=wind(1000:30000,2); power8=power(1000:30000,2); tqld8=tqld(1000:30000,2); omgls8=omgls(1000:30000,2); pitchvel8=pitchvel(1000:30000,2);
fprintf('Mean wind speed of 10 m/s ...\n')
wmean=10;init
sim('wind_turbine2')
wind10=wind(1000:30000,2); power10=power(1000:30000,2); tqld10=tqld(1000:30000,2); omgls10=omgls(1000:30000,2); pitchvel10=pitchvel(1000:30000,2);
fprintf('Mean wind speed of 12 m/s ...\n')
wmean=12;init
sim('wind_turbine2')
wind12=wind(1000:30000,2); power12=power(1000:30000,2); tqld12=tqld(1000:30000,2); omgls12=omgls(1000:30000,2); pitchvel12=pitchvel(1000:30000,2);
fprintf('Mean wind speed of 14 m/s ...\n')
wmean=14;init
sim('wind_turbine2')
wind14=wind(1000:30000,2); power14=power(1000:30000,2); tqld14=tqld(1000:30000,2); omgls14=omgls(1000:30000,2); pitchvel14=pitchvel(1000:30000,2);
fprintf('Mean wind speed of 16 m/s ...\n')
wmean=16;init
sim('wind_turbine2')
wind16=wind(1000:30000,2); power16=power(1000:30000,2); tqld16=tqld(1000:30000,2); omgls16=omgls(1000:30000,2); pitchvel16=pitchvel(1000:30000,2);
fprintf('Mean wind speed of 18 m/s ...\n')
wmean=18;init
sim('wind_turbine2')
wind18=wind(1000:30000,2); power18=power(1000:30000,2); tqld18=tqld(1000:30000,2); omgls18=omgls(1000:30000,2); pitchvel18=pitchvel(1000:30000,2);
fprintf('Mean wind speed of 20 m/s ...\n')
wmean=20;init
sim('wind_turbine2')
wind20=wind(1000:30000,2); power20=power(1000:30000,2); tqld20=tqld(1000:30000,2); omgls20=omgls(1000:30000,2); pitchvel20=pitchvel(1000:30000,2);
fprintf('Mean wind speed of 22 m/s ...\n')
wmean=22;init
sim('wind_turbine2')
wind22=wind(1000:30000,2); power22=power(1000:30000,2); tqld22=tqld(1000:30000,2); omgls22=omgls(1000:30000,2); pitchvel22=pitchvel(1000:30000,2);

wind=[wind5; wind8;wind10;wind12;wind14;wind16;wind18;wind20;wind22];
power=[power5;power8;power10;power12;power14;power16;power18;power20;power22];
tqld=[tqld5;tqld8;tqld10;tqld12;tqld14;tqld16;tqld18;tqld20;tqld22];
omgls=[omgls5;omgls8;omgls10;omgls12;omgls14;omgls16;omgls18;omgls20;omgls22];
pitchvel=[pitchvel5;pitchvel8;pitchvel10;pitchvel12;pitchvel14;pitchvel16;pitchvel18;pitchvel20;pitchvel22];
t=[0:0.02:(length(wind)-1)*0.02]';
ii=find(wind>=5); t=t(ii);wind=wind(ii);power=power(ii);tqld=tqld(ii); pitvel=pitchvel(ii);omgls=omgls(ii);

% performance plots
fprintf('Evaluating performance metrics...\n')
% collect data into time bins
i1=floor(min(t));i2=floor(max(t));
j=1;
for i=i1:20:i2-20
   ii=find(t>i & t<i+20);
   if length(ii)>0
   meanw(j)=mean(wind(ii));
   meant(j)=mean(tqld(ii));
   maxt(j)=max(tqld(ii));
   mint(j)=min(tqld(ii));
   meanp(j)=mean(power(ii));
   maxp(j)=max(power(ii));
   minp(j)=min(power(ii));
   meano(j)=mean(omgls(ii));
   maxo(j)=max(omgls(ii));
   mino(j)=min(omgls(ii));
   meanpit(j)=mean(pitvel(ii));
   stdpit(j)=std(pitvel(ii));
   j=j+1;
   end   
end

% steady state torque corresponding to each wind speed
ii1=find(meanw<=7.81);
ii2=find(meanw>7.81 & meanw<=11.85);
ii3=find(meanw>11.85);
sstq(ii1)=(Kv*meanw(ii1).^2/2/N); 
sstq(ii2)=(-(g1+N*N*g2)*wref*meanw(ii2)./meanw(ii2)+Kv*meanw(ii2).^2)/N;
sstq(ii3)=1.0e6/wref/N;
% steady state power corresponding to each wind speed
sspower(ii1)=1/(g1+N*N*g2)*(Kv*meanw(ii1).^2/2).^2; % steady state power corresponding to each wind speed
sspower(ii2)=wref*(-(g1+N*N*g2)*wref*meanw(ii2)./meanw(ii2)+Kv*meanw(ii2).^2);
sspower(ii3)=1.0e6;

% now form 1 m/s wind speed bins
for i=1:20
   V(i)=i+4;
   ii=find(meanw>i+3.5 & meanw<i+4.5);
   if length(ii)>0
    mp(i)=mean(meanp(ii)); mpx(i)=mean(maxp(ii));  sigmapx(i)=std(maxp(ii)); 
    mtq(i)=mean(meant(ii)); mtx(i)=mean(maxt(ii));  sigmatx(i)=std(maxt(ii));  mtm(i)=mean(mint(ii));sigmatm(i)=std(mint(ii));   
    mo(i)=mean(meano(ii)); mox(i)=mean(maxo(ii));  sigmaox(i)=std(maxo(ii));     
    mpit(i)=mean(stdpit(ii));  sigmapit(i)=std(stdpit(ii));     
    mstq(i)=mean(sstq(ii)); msp(i)=mean(sspower(ii));
   end   
end

figure(1),plot(meanw,meano,'or',meanw,maxo,'+r',meanw,2.72*meanw./meanw,meanw,3.264*meanw./meanw,'--'),ylabel('Rotor Speed (rad/s)'),xlabel('Wind (m/s)') 
figure(2),plot(meanw,meanp,'or',meanw,maxp,'+r',V,msp,V,mp), ylabel('Power (W)'),xlabel('Wind (m/s)')
figure(3),plot(meanw,meant,'or',meanw,maxt,'+r',meanw,mint,'+b',V,mstq,V,mtq), ylabel('Torque (Nm)'),xlabel('Wind (m/s)')
figure(4),plot(meanw,stdpit,'or',V,0.4*V-1.8,'--' ), ylabel('Pitch Vel SD (deg/s)'),xlabel('Wind (m/s)')
