% Reproduce published stability analysis results
% Note that piece-wise quadratic Lyapunov stability analysis requires PWLTools toolbox for Matlab.
%
% (c) D.J.Leith, 2002
%
% This software may be freely distributed provided it is not modified (in particular this notice must not 
% be removed) or used for commercial purposes without permission in writing.  
%
% Simplified wind turbine model
wmean=22; init

% Analysis ...
%Plant matrices
Ap=[-g2/I2 1/N/I2 0; -k1/N 0 k1; (gdash/N-gdash*k1/N/N/N/k2)/I1 -1/I1 -(g1+gdash-gdash*k1/N/N/k2)/I1];
Bp1=[-1/I2;0;0]; Bp2=[0;0;1/I1]; Bp=[Bp1 Bp2];
Cp=[1 0 0]; Dp=0;

%Controller matrices
Ald=-20; Bld=4;Cld=5;Dld=0;
Alow=[-4 -3.5 0;4 0 0;2.5 0.125 0]; Blow=[76.8;0;0];Clow=[0 0 -31.25];Dlow=0;
Bi=[1;0;0];Dlow=0;

%A1
K=(g1/N/N+g2);
ol1=ss(Ap,Bp1,Cp,Dp)*ss(Ald,Bld,Cld,Dld)*tf(-K,1); % open-loop tf in mode 2
A1=[   Ap      zeros(3,3)  Bp1*Cld;
    -Blow*Cp     Alow+Bi*Clow     zeros(3,1);
     Bld*K*Cp  zeros(1,3)     Ald];
%double check on A matrices ...
B1=[zeros(3,1);Blow;-Bld*K]; C1=[Cp zeros(1,3),0];D1=0; bode(ss(A1,B1,C1,D1),feedback(ol1,tf(1,1),-1))

%A2
K=(g1/N/N+g2);
ol2=ss(Ap,Bp1,Cp,Dp)*ss(Ald,Bld,Cld,Dld)*(ss(Alow,Blow,Clow,Dlow)-K); % open-loop tf in mode 2
%figure(1),margin(ol2); %title('mode 2 open-loop tf')
A2=[   Ap      zeros(3,3)  Bp1*Cld;
    -Blow*Cp     Alow     zeros(3,1);
     Bld*K*Cp  Bld*Clow     Ald];
%double check on A matrices ...
%B2=[zeros(3,1);Blow;-Bld*K]; C2=[Cp zeros(1,3),0];D2=0; bode(ss(A2,B2,C2,D2),feedback(ol2,tf(1,1),-1))

% A3
ol3=ss(Ap,-Bp2*Kp,Cp,Dp)*N/Kp*(ss(Alow,Blow,Clow,Dlow)-K); % open-loop tf in mode 2
%figure(2),margin(ol3); %title('mode 3 open-loop tf')
A3=[   Ap-Bp2*N*K*Cp   -Bp2*N*Clow zeros(3,1);
    -Blow*Cp     Alow     zeros(3,1);
     Bld*K*Cp  Bld*Clow     Ald ];
%double check on A matrices ...
%B3=[Bp2*N*K;Blow;0]; C3=[Cp zeros(1,3),0];D3=0;bode(ss(A3,B3,C3,D3),feedback(ol3,tf(1,1),-1))
    
% LMI based direct CQLF tests 
% A1 & A2
setlmis([]);
P=lmivar(1,[7 1]);
lmiterm([1 1 1 P], A1',1,'s');
lmiterm([2 1 1 P], A2',1,'s');
lmiterm([3 1 1 P], -1,1);
cqlf=getlmis;
[tmin,xfeas]=feasp(cqlf);
Popt12=dec2mat(cqlf,xfeas,P);
pause
% A2 & A3
setlmis([]);
P=lmivar(1,[7 1]);
lmiterm([1 1 1 P], A2',1,'s');
lmiterm([2 1 1 P], A3',1,'s');
lmiterm([3 1 1 P], -1,1);
cqlf=getlmis;
[tmin,xfeas]=feasp(cqlf);
Popt23=dec2mat(cqlf,xfeas,P);
pause
%A1, A2 & A3
setlmis([]);
P=lmivar(1,[7 1]);
lmiterm([1 1 1 P], A1',1,'s');
lmiterm([2 1 1 P], A2',1,'s');
lmiterm([3 1 1 P], A3',1,'s');
lmiterm([4 1 1 P], -1,1);
cqlf=getlmis;
[tmin,xfeas]=feasp(cqlf);
Popt123=dec2mat(cqlf,xfeas,P);
%evlmi=evallmi(cqlf,xfeas);
%[lhs1,rhs1]=showlmi(evlmi,1);
%[lhs2,rhs2]=showlmi(evlmi,2);
%[lhs3,rhs3]=showlmi(evlmi,3);

%return 

%PQLF tests ...
%A1 & A2
kappa1=K;
TRef=1e6/wref/N;
setpart('h');
options =[0,100,1e11,0,0];
dyn1=addati(A1,[],[],[],[],[]);
dyn2=addati(A2,[],[],[],[],[]);
%dyn3=addati(A3,[],[],[],[],[]);
%hp1=addhp([kappa1,0,0,0,0,1,0,-TRef]);
hp2=addhp([0 0 0 0 0 1 0 0]);
reg1=addhcell([-hp2],dyn1);
reg2=addhcell([hp2],dyn2);
%%reg2=addhcell([-hp1, hp2],dyn2);
%reg3=addhcell([hp1,hp2],dyn3);
myprt=getpart;
swtchsys=part2pwl(myprt);
[P,nl,nv]=pqstab(swtchsys,options)

return

